/*****************************************************************************
*                                                                            *
* void      Read_PCX_Picture()                                               *
*                                                                            *
******************************************************************************
*                                                                            *
* Liest ein PCX-Bild ein.                                                    *
*                                                                            *
* Folgende Modi werden verarbeitet:  5
*                                                                            *
******************************************************************************
*                                                                            *
* Erstellungsdatum: 30.09.1992                                               *
* Letzte Aenderung: 01.10.1992                                               *
*                                                                            *
******************************************************************************
*                                                                            *
* Aufruf von: ClearTextWindow(), WriteText(), Read_Int(),                    *
*                                                                            *
*****************************************************************************/

#include "const.h"

#include "prototyp.h"

#define PCX_HEADERSIZE  128

int Read_PCX_Picture(Bild_Nummer)
{
  int Anzahl;
  unsigned char PCXHeader[PCX_HEADERSIZE]; /* PCX-Bild-Header               */
  unsigned char RLC[2];                    /* Hilfsvektor fuer Run-Length-C.*/
  unsigned char Pixel;
  int PCX_fd;                              /* File-Descriptor               */
  int i, j, z, s;
  long Index;
  char Filename[LINE_LENGTH];             /* Name Ziel-Datei (.PCX)         */
  char *PCX_Ext  =".pcx";

  int NoFileFound;

  int Z_Reduktion, S_Reduktion, Z_Res, S_Res;/* Zeilen- u. Spaltenaufloesung*/
  int Spaltenzahl, S_Cut, Z_Cut;           /* zur Begrenzung der Bildgroesse*/
  unsigned char Temp_Buff[1300];           /* Hilfspuffer beim Zeilen einl. */


  struct ffblk File_Blk;                   /* Struktur des DIR-Blocks       */

  /*------------------------- PCX-Dateien auflisten ------------------------*/
  WriteText( 1, 1,"Bild im PCX-Format einlesen");
  WriteText( 1, 3,"Vorhandene Bilddateien:");

  /* Gueltige Bild-Nummer uebergeben?                                      */
  if ((Bild_Nummer<0) || (Bild_Nummer>=MAX_PICTURE)) {
     return(FALSE);
  }

  i=1;
  j=5;
  NoFileFound = findfirst("*.pcx", &File_Blk, 0);  /* alle .PCX-Dateien anz.*/
  while (!NoFileFound) {
     WriteText( i, j, File_Blk.ff_name);
     NoFileFound=findnext(&File_Blk);
     j++;
     if (j==10) {
        j= 5;
        i=i+15;
        if (i>70) NoFileFound=TRUE;
     }
  }

  /*---------------------------- Dateiname eingeben ------------------------*/
  gotoxy(1,10);
  Read_String("\nBitte Dateinamen eingeben: ", Filename);
  if (!Add_Extension(Filename, PCX_Ext)) return(FALSE);

  if ( (PCX_fd=open(Filename, O_RDONLY | O_BINARY)) == -1) {
     ShowError("Fehler beim Oeffnen der Datei.");
     return(FALSE);
  }
  read(PCX_fd, PCXHeader, PCX_HEADERSIZE );      /* liest PCX-Header und berechnet ... */

  /*------------------------- PCX-Header ueberpruefen ----------------------*/
  if ( (PCXHeader[0]!=10) || (PCXHeader[1]!=5) ||
       (PCXHeader[2]!= 1) || (PCXHeader[3]!=8) ) {
     ShowError("Falsches Datenformat.");
     return(FALSE);
  }

  /* Die Bildmasse bestimmen. Achtung bezueglich High-Low-Byte              */
  S_Res = 256*(int)PCXHeader[ 9]+(int)PCXHeader[ 8]+1;
  Z_Res = 256*(int)PCXHeader[11]+(int)PCXHeader[10]+1;

  /* Clipping der Zeilen und Spalten auf die maximal verfuegbare Groesse    */
  if (S_Res>S_RES_MAX) S_Cut=S_RES_MAX;
  else                 S_Cut=S_Res;
  S_Reduktion=S_Res/S_Cut;
  Spaltenzahl=S_Res;
  if (S_Reduktion == 1) Spaltenzahl=S_Cut;
  if (Z_Res>Z_RES_MAX) Z_Cut=Z_RES_MAX;
  else                 Z_Cut=Z_Res;
  Z_Reduktion=Z_Res/Z_Cut;

  /*------------------------ eigentliches Bild aufbauen --------------------*/
  Index = (long) (Bild_Nummer) * PIC_SIZE;
  z=0;
  while (z<Z_Res) {                    /* Bild zeilenweise aufbauen und ... */
        s=0;
        j=0;
        while (s<S_Res) {              /* ...RL-Kodierung entschluesseln.   */
              read(PCX_fd, RLC, 1);
              Anzahl=RLC[0];
              if (Anzahl > 192) {      /* Kennzeichen fuer RLC              */
                 Anzahl=Anzahl-192;
                 read(PCX_fd, RLC, 1);
                 Pixel=RLC[0];
                 for (i=0; i<Anzahl; i++) {
                     Temp_Buff[j++]=Pixel;
                     s++;
                 }
               } else {               /* ein einzelner Bildpunkt unkodiert  */
                 Temp_Buff[j++]=Anzahl;
                 s++;
               }
        }

        if ((z % Z_Reduktion) == 0)   /* groessere Bilder ggf. verkleinern  */
           for (j=0; j<Spaltenzahl; j+=S_Reduktion) Pictures[Index++]=Temp_Buff[j];
        z++;
  }
  close(PCX_fd);

  /* Genaue Bildabmessungen in Struktur speichern                           */
  Bild[Bild_Nummer].Zeilen =Z_Cut;
  Bild[Bild_Nummer].Spalten=S_Cut;

  return(TRUE);
}
